function varargout = gui_create(in1,in2)
%
% Creates structure of handles according to input.
%
% HANDLE = gui_create(GUI) or 
% [HANDLE,new_handle] = gui_create(GUI)
%
% Creates graphics objects from a structure GUI and places them into a 
% structure of handles named HANDLE.
%
% HANDLE = gui_create(GUI,HANDLE) or 
% [HANDLE,new_handle] = gui_create(GUI,HANDLE)
%
% Creates graphics objects from a structure GUI and adds them to a 
% structure of handles named HANDLE.
%
% HANDLE = gui_create(handle) or 
% [HANDLE,new_handle] = gui_create(handle)
%
% Puts a matrix of handles into a new structure of handles named HANDLE.
%
% HANDLE = gui_create(handle,HANDLE) or 
% [HANDLE,new_handle] = gui_create(handle,HANDLE)
%
% Puts a matrix of handles into an existing structure of handles named
% HANDLE.
%
% The structure HANDLE typically looks like this:
%
%      root: 0
%    figure: [1 2 3 4]
%      axes: [3.0026 4.0011 5.0011 6.0011 7.0011]
%      text: [8.0011 9.0011]
%      line: [10.0011 11.0011]
% uicontrol: [12.0011 13.0011 14.0011 15.0011 16.0011]
%
% The input structure GUI has such a form:
%
%  GUI.figure = struct( ...
%       'position', { ...
%	[0.100, 0.100, 0.300, 0.500], ...
%	[0.500, 0.100, 0.300, 0.500], ...
%	[0.100, 0.700, 0.700, 0.200]  ...
%	}, ...
%	'menubar', { 'none' }, ...
%	'toolbar', { 'none' }, ...
%	'units', { 'normalized' }, ...
%	'resize', { 'off' }, ...
%	'visible', { 'on', 'on', 'off' } ...
%	);
%
% Here instead of figure all posible graphics objects can appear:
% 
% 1: figure
% 2: axes uicontrol uimneu uicontextmenu
% 3: image light line patch rectangle surface text
%
% The hierachy in the structure must be according to the numbering (i.e.: 
% figure must given before axes, axes must be given before line, ...
%
% All possible properties and their respective values can appear in the
% structure.
%
% Lower case letters are used throughout all names of objects and
% properties.
% 
% Here, the property 'parent' is used to place objects in the proper
% context. One has to specifiy an integer number to place it in the proper
% parent objetc:
%
%  GUI.line = struct( 'parent', {2} )
%
% places the line in the second axes defined before. Finally, the property
% 'parent' in the structur HANDLE really contains the handle for the parent
% object.
%
% Author:  Winfried Kernbichler
% Version: 2.0 
% Date:    October 10, 2001
% Status:  Workes very well in all tests at the moment


if nargin == 1
  if ishandle(in1)
    handle = in1;
    HANDLE = [];
    action = 'create_handle'
  else
    GUI = in1;
    HANDLE = struct('root',0);
    action = 'create_gui';
  end
elseif nargin == 2
  if ishandle(in1)
    handle = in1;
    HANDLE = in2;
    action = 'add_handle';
  else
    GUI = in1;
    HANDLE = in2;
    action = 'add_gui';
  end
end

switch action
  case  {'create_gui', 'add_gui'}
    new_handle = []; % vector of new handles
    % Define parent objects for graphical objects
    PARENTS = struct( ...
	'figure',{'root'}, ...
	'axes',{'figure'}, ...
	'uicontrol',{'figure'}, ...
	'uimenu',{'figure'}, ...
	'uicontextmenu',{'figure'}, ...
	'image',{'axes'}, ...
	'light',{'axes'}, ...
	'line',{'axes'}, ...
    'patch',{'axes'}, ...
	'surface',{'axes'}, ...
	'text',{'axes'} ...	  
	);
    
    if strcmp(action,'create_gui')
      % Remember visible status of figures	    
      if isfield(GUI.figure,'visible')
          visible_store = {GUI.figure.visible};
      else
          visible_store = repmat({'on'},1,length(GUI.figure));
      end
      for m = 1:length(GUI.figure)
          GUI.figure = setfield(GUI.figure,{m},'visible','off');
      end
    end
    
    objects = fieldnames(GUI);      % all objects in GUI  
    for obj_l = 1:length(objects)   % cycle through objects
      obj = char(objects(obj_l));   % one object
      properties = fieldnames( eval(['GUI.',obj]) );  % all properties of object
      object_num = length( eval(['GUI.',obj]) );      % and there number
      len=0;
      if isfield(HANDLE,obj)        % determine number of entries in field
          len = eval(['length(HANDLE.',obj,')']);
      end      
      for obj_m = 1:object_num               % cycle through
	%prop_val_1 = struct('parent',0);     % structure for parent and units
	prop_val_1 = [];                     % structure for parent and units
	prop_val_2 = [];                     % structure for resize
	prop_val_3 = [];                     % structure with property and value
	for prop_l = 1:length(properties)    % through all properties
	  prop = char(properties(prop_l));   % property and value
	  value = eval(['GUI.',obj,'(',num2str(obj_m),')','.',prop]);
	  if strcmp(prop,'parent')           % find handle for parent
	    par = getfield(PARENTS,obj);
	    % handle problems
	    % try:   look first if it is entry into HANDLE
	    % catch: take it as handle 
	    warning('off');
	    try,   parhandle = eval(['HANDLE.',par,'(',num2str(value),')']);
	    catch, parhandle = value;
	    end
	    warning('on');
	    prop_val_1 = setfield(prop_val_1,prop,parhandle);
	  elseif strcmp(prop,'units') | strcmp(prop,'toolbar') % must be done before
	    prop_val_1 = setfield(prop_val_1,prop,value);
	  elseif strcmp(prop,'resize')  % must be done before
	    prop_val_2 = setfield(prop_val_2,prop,value);
	  else                               % rest of properties
	    prop_val_3 = setfield(prop_val_3,prop,value);
	  end
	end

	% set parent property if not present
	prop = 'parent';
	if ~isfield(prop_val_1,prop)
	  par = getfield(PARENTS,obj);
	  switch par
	    case 'root',   value = 0;
	    case 'figure', value = gcf;
	    case 'axes',   value = gca;
	  end
	  prop_val_1 = setfield(prop_val_1,prop,value);	      
	end
	
	com = [obj,'(prop_val_1)'];          % graphics command
	handle = eval(com);                  % evaluation of command
	if ~isempty(prop_val_2)
	  eval(['set(handle,prop_val_2)']);  % set of resize
	end
	if ~isempty(prop_val_3)
	  eval(['set(handle,prop_val_3)']);  % set of remaining properties
	end
	HANDLE = setfield(HANDLE,{1},obj,{len+obj_m},handle); % into handle Structure
	new_handle(end+1) = handle;
      end
    end

    if strcmp(action,'create_gui')
      % Set figures to correct visible status
      for m = 1:length(GUI.figure)
	set(HANDLE.figure(m),'visible',char(visible_store(m)));
      end
    end
  
  case {'create_handle','add_handle'}    % create structure from normal handles
    new_handle = handle;
    for h = 1:length(handle(:))          % go through all handles
      obj_type = get(handle(h),'type');  % get type of object 
      len = 0;
      if isfield(HANDLE,obj_type)        % determine number of entries in field
          len = eval(['length(HANDLE.',obj_type,')']);
      end
      HANDLE = setfield(HANDLE,obj_type,{len+1},handle(h)); % set field
    end
              
  otherwise
    disp('no defined')
end

varargout{1} = HANDLE;
varargout{2} = new_handle;

